<?php
/*--------------------------------------------------------------------
 Reader.php 2020-2-18
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Shop\Product\SellingUnitImage\Database\Repository\Readers;

use Doctrine\DBAL\Connection;
use Gambio\ProductImageList\Image\Entities\Image;
use Gambio\Shop\Language\ValueObjects\LanguageId;
use Gambio\Shop\Product\ValueObjects\ProductId;
use Gambio\Shop\ProductModifiers\Modifiers\ValueObjects\ModifierIdentifierCollectionInterface;
use Gambio\Shop\Product\SellingUnitImage\Database\Exceptions\CombinationDoesNotHaveAnImageListException;
use Gambio\Shop\Product\SellingUnitImage\Database\Exceptions\ImageListIsEmptyException;
use Gambio\Shop\Product\SellingUnitImage\Database\Exceptions\ProductDoesNotHaveAnImageException;
use Gambio\Shop\Product\SellingUnitImage\Database\Repository\DTO\ImageDto;
use Gambio\Shop\Product\SellingUnitImage\Database\Repository\DTO\ImageDtoCollection;

/**
 * Class Reader
 * @package Gambio\Shop\SellingUnit\ProductInformation\Services\ProductImage\Repositories
 */
class Reader implements ReaderInterface
{
    protected const IMAGE_PATH_PREFIX  = 'images/product_images/info_images/';
    protected const IMAGE_INFO_PREFIX  = 'images/product_images/info_images/';
    protected const IMAGE_POPUP_PREFIX = 'images/product_images/popup_images/';
    protected const IMAGE_THUMB_PREFIX = 'images/product_images/thumbnail_images/';
    
    protected const PRODUCTS_IMAGES_TABLE         = 'products_images';
    protected const PRODUCTS_ID_COLUMN            = 'products_images.products_id';
    protected const PRODUCTS_IMAGE_NAME_COLUMN    = 'products_images.image_name';
    protected const PRODUCTS_IMAGE_SORTING_COLUMN = 'products_images.image_nr';
    protected const PRODUCTS_IMAGE_ACTIVE_COLUMN  = 'products_images.gm_show_image';
    
    /**
     * @var Connection
     */
    protected $connection;
    
    
    /**
     * ProductImageReader constructor.
     *
     * @param Connection $connection
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }
    
    
    /**
     * @inheritDoc
     */
    public function getMainProductImage(ProductId $id, LanguageId $languageId): ImageDto
    {
        $productId = $id->value();
        $result    = $this->connection->createQueryBuilder()
                                      ->select('p.products_image, 0 as image_nr, pd.gm_alt_text')
                                      ->from('products', 'p')
                                      ->leftJoin('p',
                                                 'products_description',
                                                 'pd',
                                                 'pd.products_id = p.products_id and pd.language_id = '
                                                 . $languageId->value())
                                      ->where('p.products_id = ' . $id->value())
                                      ->andWhere("trim(coalesce(products_image, '')) <> ''")
                                      ->andWhere('gm_show_image = 1')
                                      ->execute();
        
        if ($result->rowCount() !== 0) {
            
            $row = $result->fetch(0);
            
            return new ImageDto(self::IMAGE_PATH_PREFIX . $row['products_image'],
                                self::IMAGE_INFO_PREFIX . $row['products_image'],
                                self::IMAGE_POPUP_PREFIX . $row['products_image'],
                                self::IMAGE_THUMB_PREFIX . $row['products_image'],
                                $row['gm_alt_text'] ?? '',
                                (int)$row['image_nr']);
        }
        
        throw new ProductDoesNotHaveAnImageException('Product:' . $productId . ' has no active Image in');
    }
    
    
    /**
     * @inheritDoc
     */
    public function getProductImages(ProductId $id, LanguageId $languageId): ImageDtoCollection
    {
        $result      = new ImageDtoCollection();
        $productId   = $id->value();
        $queryResult = $this->connection->createQueryBuilder()
                                        ->select('products_images.image_name,products_images.image_nr, alt.gm_alt_text')
                                        ->from(self::PRODUCTS_IMAGES_TABLE)
                                        ->leftJoin(self::PRODUCTS_IMAGES_TABLE,
                                                   'gm_prd_img_alt',
                                                   'alt',
                                                   'alt.image_id = products_images.image_id
                                                  and alt.language_id = ' . $languageId->value())
                                        ->where(self::PRODUCTS_ID_COLUMN . ' = ' . $id->value())
                                        ->andWhere(self::PRODUCTS_IMAGE_ACTIVE_COLUMN . ' = 1')
                                        ->orderBy(self::PRODUCTS_IMAGE_SORTING_COLUMN)
                                        ->execute();
        
        foreach ($queryResult as $row) {
            $result[] = new ImageDto(self::IMAGE_PATH_PREFIX . $row['image_name'],
                                     self::IMAGE_INFO_PREFIX . $row['image_name'],
                                     self::IMAGE_POPUP_PREFIX . $row['image_name'],
                                     self::IMAGE_THUMB_PREFIX . $row['image_name'],
                                     $row['gm_alt_text'] ?? '',
                                     (int)$row['image_nr']);
        }
        
        if (!count($result)) {
            throw new ProductDoesNotHaveAnImageException('Product:' . $productId . ' has no active Image in');
        }
        
        return $result;
    }
}